@@ -1,9 +1,10 @@
 import os
 
-def ReadConfig(file_name):
-    CONFIG_FILE = os.path.abspath(file_name)
-    C = {}
-    with open(CONFIG_FILE, 'r') as f:
+
+def read_config(file_name):
+    config_file = os.path.abspath(file_name)
+    config = {}
+    with open(config_file, 'r') as f:
         for l in f.readlines():
             l = l.split('#')[0]
             l = l.strip()
@@ -14,6 +15,6 @@ def ReadConfig(file_name):
                 key, value = key.strip(), int(value.strip())
             except:
                 continue
-            C[key] = value
+            config[key] = value
 
-    return C
+    return config
