@@ -1,8 +1,10 @@
-#!/usr/bin/env python
+"""from https://github.com/wsilva32/poly_decomp.py"""
 
 from __future__ import division
+
 import math
 
+
 def lineInt(l1, l2, precision=0):
     """Compute the intersection between two lines.
 
@@ -14,7 +16,7 @@ def lineInt(l1, l2, precision=0):
     Returns:
     The intersection point
     """
-    i = [0, 0] # point
+    i = [0, 0]  # point
     a1 = l1[1][1] - l1[0][1]
     b1 = l1[0][0] - l1[1][0]
     c1 = a1 * l1[0][0] + b1 * l1[0][1]
@@ -22,11 +24,12 @@ def lineInt(l1, l2, precision=0):
     b2 = l2[0][0] - l2[1][0]
     c2 = a2 * l2[0][0] + b2 * l2[0][1]
     det = a1 * b2 - a2 * b1
-    if not scalar_eq(det, 0, precision): # lines are not parallel
+    if not scalar_eq(det, 0, precision):  # lines are not parallel
         i[0] = (b2 * c1 - b1 * c2) / det
         i[1] = (a1 * c2 - a2 * c1) / det
     return i
 
+
 def lineSegmentsIntersect(p1, p2, q1, q2):
     """Checks if two line segments intersect.
 
@@ -45,7 +48,7 @@ def lineSegmentsIntersect(p1, p2, q1, q2):
     db = q2[1] - q1[1]
 
     # segments are parallel
-    if (da*dy - db*dx) == 0:
+    if (da * dy - db * dx) == 0:
         return False
 
     s = (dx * (q1[1] - p1[1]) + dy * (p1[0] - q1[0])) / (da * dy - db * dx)
@@ -53,6 +56,7 @@ def lineSegmentsIntersect(p1, p2, q1, q2):
 
     return s >= 0 and s <= 1 and t >= 0 and t <= 1
 
+
 def triangleArea(a, b, c):
     """Calculates the area of a triangle spanned by three points.
     Note that the area will be negative if the points are not given in counter-clockwise order.
@@ -65,20 +69,25 @@ def triangleArea(a, b, c):
     Returns:
     Area of triangle
     """
-    return ((b[0] - a[0])*(c[1] - a[1]))-((c[0] - a[0])*(b[1] - a[1]))
+    return ((b[0] - a[0]) * (c[1] - a[1])) - ((c[0] - a[0]) * (b[1] - a[1]))
+
 
 def isLeft(a, b, c):
     return triangleArea(a, b, c) > 0
 
+
 def isLeftOn(a, b, c):
     return triangleArea(a, b, c) >= 0
 
+
 def isRight(a, b, c):
     return triangleArea(a, b, c) < 0
 
+
 def isRightOn(a, b, c):
     return triangleArea(a, b, c) <= 0
 
+
 def collinear(a, b, c, thresholdAngle=0):
     """Checks if three points are collinear.
 
@@ -97,22 +106,24 @@ def collinear(a, b, c, thresholdAngle=0):
         ab = [None] * 2
         bc = [None] * 2
 
-        ab[0] = b[0]-a[0]
-        ab[1] = b[1]-a[1]
-        bc[0] = c[0]-b[0]
-        bc[1] = c[1]-b[1]
+        ab[0] = b[0] - a[0]
+        ab[1] = b[1] - a[1]
+        bc[0] = c[0] - b[0]
+        bc[1] = c[1] - b[1]
 
-        dot = ab[0]*bc[0] + ab[1]*bc[1]
-        magA = math.sqrt(ab[0]*ab[0] + ab[1]*ab[1])
-        magB = math.sqrt(bc[0]*bc[0] + bc[1]*bc[1])
-        angle = math.acos(dot/(magA*magB))
+        dot = ab[0] * bc[0] + ab[1] * bc[1]
+        magA = math.sqrt(ab[0] * ab[0] + ab[1] * ab[1])
+        magB = math.sqrt(bc[0] * bc[0] + bc[1] * bc[1])
+        angle = math.acos(dot / (magA * magB))
         return angle < thresholdAngle
 
+
 def sqdist(a, b):
     dx = b[0] - a[0]
     dy = b[1] - a[1]
     return dx * dx + dy * dy
 
+
 def polygonAt(polygon, i):
     """Gets a vertex at position i on the polygon.
     It does not matter if i is out of bounds.
@@ -127,6 +138,7 @@ def polygonAt(polygon, i):
     s = len(polygon)
     return polygon[i % s]
 
+
 def polygonClear(polygon):
     """Clears the polygon data
 
@@ -136,6 +148,7 @@ def polygonClear(polygon):
     """
     del polygon[:]
 
+
 def polygonAppend(polygon, poly, start, end):
     """Grabs points at indicies `start` to `end` from `poly`
     and appends them to `polygon`
@@ -150,6 +163,7 @@ def polygonAppend(polygon, poly, start, end):
     for i in range(start, end):
         polygon.append(poly[i])
 
+
 def polygonMakeCCW(polygon):
     """Makes sure that the polygon vertices are ordered counter-clockwise.
 
@@ -169,6 +183,7 @@ def polygonMakeCCW(polygon):
     if not isLeft(polygonAt(polygon, br - 1), polygonAt(polygon, br), polygonAt(polygon, br + 1)):
         polygonReverse(polygon)
 
+
 def polygonReverse(polygon):
     """Reverses the vertices in the polygon.
 
@@ -178,6 +193,7 @@ def polygonReverse(polygon):
     """
     polygon.reverse()
 
+
 def polygonIsReflex(polygon, i):
     """Checks if a point in the polygon is a reflex point.
 
@@ -191,6 +207,7 @@ def polygonIsReflex(polygon, i):
     """
     return isRight(polygonAt(polygon, i - 1), polygonAt(polygon, i), polygonAt(polygon, i + 1))
 
+
 def polygonCanSee(polygon, a, b):
     """Checks if two vertices in the polygon can see each other.
 
@@ -204,28 +221,31 @@ def polygonCanSee(polygon, a, b):
 
     """
 
-    l1 = [None]*2
-    l2 = [None]*2
+    l1 = [None] * 2
+    l2 = [None] * 2
 
-    if isLeftOn(polygonAt(polygon, a + 1), polygonAt(polygon, a), polygonAt(polygon, b)) and isRightOn(polygonAt(polygon, a - 1), polygonAt(polygon, a), polygonAt(polygon, b)):
+    if isLeftOn(polygonAt(polygon, a + 1), polygonAt(polygon, a), polygonAt(polygon, b)) and isRightOn(
+            polygonAt(polygon, a - 1), polygonAt(polygon, a), polygonAt(polygon, b)):
         return False
 
     dist = sqdist(polygonAt(polygon, a), polygonAt(polygon, b))
-    for i in range(0, len(polygon)): # for each edge
-        if (i + 1) % len(polygon) == a or i == a: # ignore incident edges
+    for i in range(0, len(polygon)):  # for each edge
+        if (i + 1) % len(polygon) == a or i == a:  # ignore incident edges
             continue
 
-        if isLeftOn(polygonAt(polygon, a), polygonAt(polygon, b), polygonAt(polygon, i + 1)) and isRightOn(polygonAt(polygon, a), polygonAt(polygon, b), polygonAt(polygon, i)): # if diag intersects an edge
+        if isLeftOn(polygonAt(polygon, a), polygonAt(polygon, b), polygonAt(polygon, i + 1)) and isRightOn(
+                polygonAt(polygon, a), polygonAt(polygon, b), polygonAt(polygon, i)):  # if diag intersects an edge
             l1[0] = polygonAt(polygon, a)
             l1[1] = polygonAt(polygon, b)
             l2[0] = polygonAt(polygon, i)
             l2[1] = polygonAt(polygon, i + 1)
             p = lineInt(l1, l2)
-            if sqdist(polygonAt(polygon, a), p) < dist: # if edge is blocking visibility to b
+            if sqdist(polygonAt(polygon, a), p) < dist:  # if edge is blocking visibility to b
                 return False
 
     return True
 
+
 def polygonCopy(polygon, i, j, targetPoly=None):
     """Copies the polygon from vertex i to vertex j to targetPoly.
 
@@ -243,12 +263,12 @@ def polygonCopy(polygon, i, j, targetPoly=None):
     polygonClear(p)
     if i < j:
         # Insert all vertices from i to j
-        for k in range(i, j+1):
+        for k in range(i, j + 1):
             p.append(polygon[k])
 
     else:
         # Insert vertices 0 to j
-        for k in range(0, j+1):
+        for k in range(0, j + 1):
             p.append(polygon[k])
 
         # Insert vertices i to end
@@ -257,6 +277,7 @@ def polygonCopy(polygon, i, j, targetPoly=None):
 
     return p
 
+
 def polygonGetCutEdges(polygon):
     """Decomposes the polygon into convex pieces.
     Note that this algorithm has complexity O(N^4) and will be very slow for polygons with many vertices.
@@ -291,6 +312,7 @@ def polygonGetCutEdges(polygon):
 
     return mins
 
+
 def polygonDecomp(polygon):
     """Decomposes the polygon into one or more convex sub-polygons.
 
@@ -307,6 +329,7 @@ def polygonDecomp(polygon):
     else:
         return [polygon]
 
+
 def polygonSlice(polygon, cutEdges):
     """Slices the polygon given one or more cut edges. If given one, this function will return two polygons (false on failure). If many, an array of polygons.
 
@@ -321,7 +344,8 @@ def polygonSlice(polygon, cutEdges):
     if len(cutEdges) == 0:
         return [polygon]
 
-    if isinstance(cutEdges, list) and len(cutEdges) != 0 and isinstance(cutEdges[0], list) and len(cutEdges[0]) == 2 and isinstance(cutEdges[0][0], list):
+    if isinstance(cutEdges, list) and len(cutEdges) != 0 and isinstance(cutEdges[0], list) and len(
+            cutEdges[0]) == 2 and isinstance(cutEdges[0][0], list):
 
         polys = [polygon]
 
@@ -333,7 +357,7 @@ def polygonSlice(polygon, cutEdges):
                 result = polygonSlice(poly, cutEdge)
                 if result:
                     # Found poly! Cut and quit
-                    del polys[j:j+1]
+                    del polys[j:j + 1]
                     polys.extend((result[0], result[1]))
                     break
 
@@ -342,8 +366,8 @@ def polygonSlice(polygon, cutEdges):
 
         # Was given one edge
         cutEdge = cutEdges
-        i = polygon.index(cutEdge[0])
-        j = polygon.index(cutEdge[1])
+        i = polygon._index(cutEdge[0])
+        j = polygon._index(cutEdge[1])
 
         if i != -1 and j != -1:
             return [polygonCopy(polygon, i, j),
@@ -351,6 +375,7 @@ def polygonSlice(polygon, cutEdges):
         else:
             return False
 
+
 def polygonIsSimple(polygon):
     """Checks that the line segments of this polygon do not intersect each other.
 
@@ -366,18 +391,19 @@ def polygonIsSimple(polygon):
     """
     path = polygon
     # Check
-    for i in range(0,len(path)-1):
-        for j in range(0, i-1):
-            if lineSegmentsIntersect(path[i], path[i+1], path[j], path[j+1]):
+    for i in range(0, len(path) - 1):
+        for j in range(0, i - 1):
+            if lineSegmentsIntersect(path[i], path[i + 1], path[j], path[j + 1]):
                 return False
 
     # Check the segment between the last and the first point to all others
-    for i in range(1,len(path)-2):
-        if lineSegmentsIntersect(path[0], path[len(path)-1], path[i], path[i+1]):
+    for i in range(1, len(path) - 2):
+        if lineSegmentsIntersect(path[0], path[len(path) - 1], path[i], path[i + 1]):
             return False
 
     return True
 
+
 def getIntersectionPoint(p1, p2, q1, q2, delta=0):
     """Gets the intersection point 
 
@@ -405,6 +431,7 @@ def getIntersectionPoint(p1, p2, q1, q2, delta=0):
     else:
         return [0, 0]
 
+
 def polygonQuickDecomp(polygon, result=None, reflexVertices=None, steinerPoints=None, delta=25, maxlevel=100, level=0):
     """Quickly decompose the Polygon into convex sub-polygons.
 
@@ -428,16 +455,16 @@ def polygonQuickDecomp(polygon, result=None, reflexVertices=None, steinerPoints=
 
     upperInt = [0, 0]
     lowerInt = [0, 0]
-    p = [0, 0]         # Points
+    p = [0, 0]  # Points
     upperDist = 0
     lowerDist = 0
     d = 0
-    closestDist = 0 # scalars
+    closestDist = 0  # scalars
     upperIndex = 0
     lowerIndex = 0
-    closestIndex = 0 # integers
+    closestIndex = 0  # integers
     lowerPoly = []
-    upperPoly = [] # polygons
+    upperPoly = []  # polygons
     poly = polygon
     v = polygon
 
@@ -446,7 +473,7 @@ def polygonQuickDecomp(polygon, result=None, reflexVertices=None, steinerPoints=
 
     level += 1
     if level > maxlevel:
-        print("quickDecomp: max level ("+str(maxlevel)+") reached.")
+        print("quickDecomp: max level (" + str(maxlevel) + ") reached.")
         return result
 
     for i in range(0, len(polygon)):
@@ -456,17 +483,22 @@ def polygonQuickDecomp(polygon, result=None, reflexVertices=None, steinerPoints=
             lowerDist = float('inf')
 
             for j in range(0, len(polygon)):
-                if isLeft(polygonAt(poly, i - 1), polygonAt(poly, i), polygonAt(poly, j)) and isRightOn(polygonAt(poly, i - 1), polygonAt(poly, i), polygonAt(poly, j - 1)): # if line intersects with an edge
-                    p = getIntersectionPoint(polygonAt(poly, i - 1), polygonAt(poly, i), polygonAt(poly, j), polygonAt(poly, j - 1)) # find the point of intersection
-                    if isRight(polygonAt(poly, i + 1), polygonAt(poly, i), p): # make sure it's inside the poly
+                if isLeft(polygonAt(poly, i - 1), polygonAt(poly, i), polygonAt(poly, j)) and isRightOn(
+                        polygonAt(poly, i - 1), polygonAt(poly, i),
+                        polygonAt(poly, j - 1)):  # if line intersects with an edge
+                    p = getIntersectionPoint(polygonAt(poly, i - 1), polygonAt(poly, i), polygonAt(poly, j),
+                                             polygonAt(poly, j - 1))  # find the point of intersection
+                    if isRight(polygonAt(poly, i + 1), polygonAt(poly, i), p):  # make sure it's inside the poly
                         d = sqdist(poly[i], p)
-                        if d < lowerDist: # keep only the closest intersection
+                        if d < lowerDist:  # keep only the closest intersection
                             lowerDist = d
                             lowerInt = p
                             lowerIndex = j
 
-                if isLeft(polygonAt(poly, i + 1), polygonAt(poly, i), polygonAt(poly, j + 1)) and isRightOn(polygonAt(poly, i + 1), polygonAt(poly, i), polygonAt(poly, j)):
-                    p = getIntersectionPoint(polygonAt(poly, i + 1), polygonAt(poly, i), polygonAt(poly, j), polygonAt(poly, j + 1))
+                if isLeft(polygonAt(poly, i + 1), polygonAt(poly, i), polygonAt(poly, j + 1)) and isRightOn(
+                        polygonAt(poly, i + 1), polygonAt(poly, i), polygonAt(poly, j)):
+                    p = getIntersectionPoint(polygonAt(poly, i + 1), polygonAt(poly, i), polygonAt(poly, j),
+                                             polygonAt(poly, j + 1))
                     if isLeft(polygonAt(poly, i - 1), polygonAt(poly, i), p):
                         d = sqdist(poly[i], p)
                         if d < upperDist:
@@ -476,37 +508,37 @@ def polygonQuickDecomp(polygon, result=None, reflexVertices=None, steinerPoints=
 
             # if there are no vertices to connect to, choose a point in the middle
             if lowerIndex == (upperIndex + 1) % len(polygon):
-                #print("Case 1: Vertex("+str(i)+"), lowerIndex("+str(lowerIndex)+"), upperIndex("+str(upperIndex)+"), poly.size("+str(len(polygon))+")")
+                # print("Case 1: Vertex("+str(i)+"), lowerIndex("+str(lowerIndex)+"), upperIndex("+str(upperIndex)+"), poly.size("+str(len(polygon))+")")
                 p[0] = (lowerInt[0] + upperInt[0]) / 2
                 p[1] = (lowerInt[1] + upperInt[1]) / 2
                 steinerPoints.append(p)
 
                 if i < upperIndex:
-                    #lowerPoly.insert(lowerPoly.end(), poly.begin() + i, poly.begin() + upperIndex + 1)
-                    polygonAppend(lowerPoly, poly, i, upperIndex+1)
+                    # lowerPoly.insert(lowerPoly.end(), poly.begin() + i, poly.begin() + upperIndex + 1)
+                    polygonAppend(lowerPoly, poly, i, upperIndex + 1)
                     lowerPoly.append(p)
                     upperPoly.append(p)
                     if lowerIndex != 0:
-                        #upperPoly.insert(upperPoly.end(), poly.begin() + lowerIndex, poly.end())
+                        # upperPoly.insert(upperPoly.end(), poly.begin() + lowerIndex, poly.end())
                         polygonAppend(upperPoly, poly, lowerIndex, len(poly))
 
-                    #upperPoly.insert(upperPoly.end(), poly.begin(), poly.begin() + i + 1)
-                    polygonAppend(upperPoly, poly, 0, i+1)
+                    # upperPoly.insert(upperPoly.end(), poly.begin(), poly.begin() + i + 1)
+                    polygonAppend(upperPoly, poly, 0, i + 1)
                 else:
                     if i != 0:
-                        #lowerPoly.insert(lowerPoly.end(), poly.begin() + i, poly.end())
+                        # lowerPoly.insert(lowerPoly.end(), poly.begin() + i, poly.end())
                         polygonAppend(lowerPoly, poly, i, len(poly))
 
-                    #lowerPoly.insert(lowerPoly.end(), poly.begin(), poly.begin() + upperIndex + 1)
-                    polygonAppend(lowerPoly, poly, 0, upperIndex+1)
+                    # lowerPoly.insert(lowerPoly.end(), poly.begin(), poly.begin() + upperIndex + 1)
+                    polygonAppend(lowerPoly, poly, 0, upperIndex + 1)
                     lowerPoly.append(p)
                     upperPoly.append(p)
-                    #upperPoly.insert(upperPoly.end(), poly.begin() + lowerIndex, poly.begin() + i + 1)
-                    polygonAppend(upperPoly, poly, lowerIndex, i+1)
+                    # upperPoly.insert(upperPoly.end(), poly.begin() + lowerIndex, poly.begin() + i + 1)
+                    polygonAppend(upperPoly, poly, lowerIndex, i + 1)
 
             else:
                 # connect to the closest point within the triangle
-                #print("Case 2: Vertex("+str(i)+"), closestIndex("+str(closestIndex)+"), poly.size("+str(len(polygon))+")\n")
+                # print("Case 2: Vertex("+str(i)+"), closestIndex("+str(closestIndex)+"), poly.size("+str(len(polygon))+")\n")
 
                 if lowerIndex > upperIndex:
                     upperIndex += len(polygon)
@@ -516,25 +548,26 @@ def polygonQuickDecomp(polygon, result=None, reflexVertices=None, steinerPoints=
                 if upperIndex < lowerIndex:
                     return result
 
-                for j in range(lowerIndex, upperIndex+1):
-                    if isLeftOn(polygonAt(poly, i - 1), polygonAt(poly, i), polygonAt(poly, j)) and isRightOn(polygonAt(poly, i + 1), polygonAt(poly, i), polygonAt(poly, j)):
+                for j in range(lowerIndex, upperIndex + 1):
+                    if isLeftOn(polygonAt(poly, i - 1), polygonAt(poly, i), polygonAt(poly, j)) and isRightOn(
+                            polygonAt(poly, i + 1), polygonAt(poly, i), polygonAt(poly, j)):
                         d = sqdist(polygonAt(poly, i), polygonAt(poly, j))
                         if d < closestDist:
                             closestDist = d
                             closestIndex = j % len(polygon)
 
                 if i < closestIndex:
-                    polygonAppend(lowerPoly, poly, i, closestIndex+1)
+                    polygonAppend(lowerPoly, poly, i, closestIndex + 1)
                     if closestIndex != 0:
                         polygonAppend(upperPoly, poly, closestIndex, len(v))
 
-                    polygonAppend(upperPoly, poly, 0, i+1)
+                    polygonAppend(upperPoly, poly, 0, i + 1)
                 else:
                     if i != 0:
                         polygonAppend(lowerPoly, poly, i, len(v))
 
-                    polygonAppend(lowerPoly, poly, 0, closestIndex+1)
-                    polygonAppend(upperPoly, poly, closestIndex, i+1)
+                    polygonAppend(lowerPoly, poly, 0, closestIndex + 1)
+                    polygonAppend(upperPoly, poly, closestIndex, i + 1)
 
             # solve smallest poly first
             if len(lowerPoly) < len(upperPoly):
@@ -550,6 +583,7 @@ def polygonQuickDecomp(polygon, result=None, reflexVertices=None, steinerPoints=
 
     return result
 
+
 def polygonRemoveCollinearPoints(polygon, precision=0):
     """Remove collinear points in the polygon.
 
@@ -564,14 +598,15 @@ def polygonRemoveCollinearPoints(polygon, precision=0):
     num = 0
     i = len(polygon) - 1
     while len(polygon) > 3 and i >= 0:
-    #(var i=polygon.length-1; polygon.length>3 && i>=0; --i){
-        if collinear(polygonAt(polygon, i - 1), polygonAt(polygon, i), polygonAt(polygon, i+1), precision):
+        # (var i=polygon.length-1; polygon.length>3 && i>=0; --i){
+        if collinear(polygonAt(polygon, i - 1), polygonAt(polygon, i), polygonAt(polygon, i + 1), precision):
             # Remove the middle point
-            del polygon[i % len(polygon):(i % len(polygon))+1]
+            del polygon[i % len(polygon):(i % len(polygon)) + 1]
             num += 1
         i -= 1
     return num
 
+
 def scalar_eq(a, b, precision=0):
     """Check if two scalars are equal.
 
