@@ -1,6 +1,8 @@
+"""From https://github.com/sseemayer/Py2D"""
+
+import itertools
 import math
 from collections import defaultdict
-import itertools
 
 
 class Vector(object):
@@ -999,7 +1001,7 @@ class Polygon(object):
             holes.remove(closest_hole)
             if Polygon.is_clockwise_s(closest_hole): closest_hole = closest_hole.clone_ccw()
 
-            i = closest_hole.points.index(d_a)
+            i = closest_hole.points._index(d_a)
             j = p.index(d_b)
 
             extension = [d_b] + closest_hole.points[i:] + closest_hole.points[:i + 1]
@@ -1168,7 +1170,7 @@ class Polygon(object):
         # filter intersection points that are boundary points
         for int_point in set(filter(lambda x: x in pts, intersections)):
 
-            i = pts.index(int_point)
+            i = pts._index(int_point)
             prv = pts[i - 1]
             nxt = pts[(i + 1) % len(pts)]
 
